HC = require 'hardoncollider'

function love.load()

	win = love.graphics.setMode(1024,800)
	
	text = {}
	
	---- Collision library ----
	HC.init(500, on_collide)
	
	spaceship_sprite = love.graphics.newImage("spaceship.png")

	spaceship = {	
		x = 512,
		y = 400,
		speed = 400,
		rotation_speed = 8,
		angle = 0,
		hc_shape = HC.addRectangle( 512 - spaceship_sprite:getWidth()/2,
									400 - spaceship_sprite:getHeight()/2,
									spaceship_sprite:getWidth(), spaceship_sprite:getHeight() ),
		hp = 50,
		index = 0
	}
	
	HC.addToGroup( "player_team", spaceship.hc_shape )
	
	dx = 0
	dy = 0
	
	spaceship.hc_shape.parent = spaceship
   
	gun = {
		x = spaceship.x + spaceship_sprite:getWidth()/2,
		y = spaceship.y
	};
	
	mouse = {};
	
	game_level = 1
	
	---- Manage bullets ----
	ammos = {};
	ammo_sprite = love.graphics.newImage("ammo.png")
	ammo_limit = 500
	ammo_timer = 0
	ammo_interval = 0.1
	ammo_default_speed = 700
	weapon_level = 1
	
	---- Manage stars ----
	stars = {};
	star_sprite = love.graphics.newImage("star.png")
	star_limit = 100
	star_timer = 0
	star_interval = 0.5
	star_default_speed = 400
		
		-- Throws in some stars to populate the screen before starting the game
	for i=1, math.random(50,70) do
		local star = {}
		star.x = math.random(0,1024)
		star.y = math.random(0,800)
		star.size_factor = math.random(20,250)/100
		star.speed = star_default_speed
		table.insert(stars,star)
	end
	
	---- Manage enemies ----
	enemies = {};
	enemy_sprite = love.graphics.newImage("hamster.png")
	enemy_limit = 500
	enemy_timer = 0
	enemy_interval = 0.2
	enemy_default_speed = 400
	
end

function love.update(dt)
	get_keys(dt)	
	update_stars(dt)
	update_ammo(dt)
	update_spaceship(dt)
	update_enemies(dt)	
	
	-- check for collisions
    HC.update(dt)
	
	while #text > 5 do
        table.remove(text, 1)
    end
	
	if (spaceship.hp <= 0) then				
		love.timer.sleep(500)
		restart()
		love.timer.sleep(500)
	end
end

function love.draw()

	for i = 1,#text do
        love.graphics.setColor(255,255,255, 255 - (i-1) * 6)
        love.graphics.print(text[#text - (i-1)], 600, i * 15)
    end

	love.graphics.print("Bullet Time : "..ammo_interval.."",10,15)
	love.graphics.print("Stars# : "..#stars.."",10,30)
	love.graphics.print("Ammos# : "..#ammos.."",10,45)
	love.graphics.print("Enemies# : "..#enemies.."",10,60)
	love.graphics.print("Shapes# : "..HC.getShapeDim().."",10,75)
	--love.graphics.print("Spaceship Angle : "..spaceship.angle*180/math.pi.."",10,30)
	--love.graphics.print("Cos Angle : "..math.cos( spaceship.angle ).."",10,45)
	--love.graphics.print("Sin Angle : "..math.sin( spaceship.angle ).."",10,60)
	--love.graphics.print("Tan Angle : "..math.tan( spaceship.angle ).."",10,75)
	love.graphics.print("Weapon Level : "..weapon_level.."",10,90)
	--love.graphics.print("Mouse x : "..mouse.x.."",10,105)
	--love.graphics.print("Mouse y : "..mouse.y.."",10,120)
	
	for i = 1, #ammos do
		if ammos[i] ~= nil then
			--ammos[i].hc_shape:draw('fill')
			love.graphics.draw( ammo_sprite, ammos[i].x, ammos[i].y, ammos[i].angle, 1, 1, 
			ammo_sprite:getWidth() / 2, ammo_sprite:getHeight() / 2 )			
		end
	end
	
	for i=1, #stars do
		if stars[i] ~= nil then
			love.graphics.draw( star_sprite, stars[i].x, stars[i].y, 0, stars[i].size_factor/4)
		end
	end
	
	for i=1, #enemies do
		if enemies[i] ~= nil then
			--enemies[i].hc_shape:draw('fill',50)
			love.graphics.draw( enemy_sprite, enemies[i].x, enemies[i].y, 0, enemies[i].size_factor)			
		end
	end
	
	if spaceship ~= nil then
		--spaceship.hc_shape:draw('fill')
		love.graphics.draw( spaceship_sprite, spaceship.x, spaceship.y, spaceship.angle,
							1, 1, spaceship_sprite:getWidth() / 2, spaceship_sprite:getHeight() / 2 )			
	end
						
	--love.graphics.rectangle( 'fill', spaceship.x, spaceship.y, 3, 3 )
	--love.graphics.rectangle( 'fill', gun.x, gun.y, 3, 3 )
end

function love.keypressed(k)
	if love.keyboard.isDown( "q" ) or love.keyboard.isDown( "escape" ) then
		love.event.push('q')
	end
	
	if love.keyboard.isDown( "u" ) then
		weapon_level = weapon_level + 1
	end
	
	if love.keyboard.isDown( "p" ) then
		weapon_level = weapon_level - 1
	end
end

function restart()
	for k in pairs (enemies) do
		if enemies[k].hc_space then
			HC.removeFromGroup("enemy_team", enemies[k].hc_space)	
			HC.remove( enemies[k].hc_space )
		end
		enemies[k] = nil
	end
	
	for k in pairs (ammos) do		
		if ammos[k].hc_space then
			HC.removeFromGroup("player_team", ammos[k].hc_space)
			HC.remove( ammos[k].hc_space )
		end
		ammos[k] = nil
	end
		
	love.graphics.setBackgroundColor( 0, 0, 0 )
	
	spaceship.x = 512
	spaceship.y = 700
	spaceship.hp = 50
end

function get_keys(dt)	

	if spaceship == nil then
		return
	end
	
	dx = dx * 0.7
	dy = dy * 0.7
	
	if love.keyboard.isDown( "right" ) then
		dx = spaceship.speed
	elseif love.keyboard.isDown( "left" ) then
		dx = - spaceship.speed
	end

	if love.keyboard.isDown( "down" ) then
		dy = spaceship.speed
	elseif love.keyboard.isDown( "up" ) then
		dy = - spaceship.speed
	end
	
	spaceship.x = spaceship.x + dx*dt
	spaceship.y = spaceship.y + dy*dt
	
	spaceship.hc_shape:moveTo( spaceship.x, spaceship.y )
	
end

function update_stars(dt)
	for i=1, #stars do
		if stars[i] ~= nil then
			if stars[i].y > 900 then
				table.remove(stars,i)
			else
				stars[i].y = stars[i].y+(stars[i].speed*dt*(stars[i].size_factor/2))
			end
		end		
	end
	
	if star_timer >= star_interval then
		for i=1, math.random(12,20) do
			local star = {}
			star.x = math.random(0,1024)
			star.y = math.random(100,200)*-1
			star.size_factor = math.random(20,250)/100
			star.speed = star_default_speed
			table.insert(stars,star)
		end
		star_timer = 0
	else
		star_timer = star_timer+dt
	end
	
	if #stars > star_limit then
		table.remove(stars,1)
	end
end

function update_ammo(dt)
	if ( ammo_timer < ammo_interval ) then
		ammo_timer = ammo_timer + dt
	end
	
	if #ammos > ammo_limit then
		HC.removeFromGroup( ammos[1].hc_shape )
		HC.remove( ammos[1].hc_shape )
		table.remove(ammos,1)
	end
	
	for i = 1, #ammos do
		if ammos[i] ~= nil then
			if ( ammos[i].x > 1100
				 or ammos[i].x < -100
				 or ammos[i].y > 900
				 or ammos[i].y < -100
				 or ammos[i].hc_shape == nil) then
				if ammos[i].hc_shape  then
					HC.removeFromGroup( ammos[i].hc_shape )
					HC.remove( ammos[i].hc_shape )
				end
				table.remove(ammos,i)
			else			 
				ammos[i].x = ammos[i].x + ammos[i].speed * math.cos( ammos[i].angle ) * dt
				ammos[i].y = ammos[i].y + ammos[i].speed * math.sin( ammos[i].angle ) * dt
				ammos[i].hc_shape:setRotation( ammos[i].angle )
				ammos[i].hc_shape:moveTo (ammos[i].x, ammos[i].y)
			end
		end
	end
	
	if love.mouse.isDown( "l" ) then
		if ( ammo_timer >= ammo_interval ) then
			 ammo_timer = 0
			for i = 0, weapon_level-1 do
				local ammo = {}
				ammo.x = gun.x
				ammo.y = gun.y
				ammo.angle = spaceship.angle
				if weapon_level ~= 1 then
					ammo.angle = ammo.angle + ( i*30/(weapon_level-1) - 15 )*math.pi/180
				end
				ammo.hc_shape = HC.addRectangle( ammo.x - ammo_sprite:getWidth()/2,
												 ammo.y - ammo_sprite:getHeight()/2,
												 ammo_sprite:getWidth(), 
												 ammo_sprite:getHeight() )
				ammo.hc_shape.sprite = ammo_sprite
				ammo.hc_shape.parent = ammo
				ammo.speed = ammo_default_speed
				HC.addToGroup( "player_team", ammo.hc_shape )
				table.insert(ammos,ammo)				
			end
		end
	end
	
	for i=1, #ammos do
		ammos[i].index = i
	end
end

function update_spaceship(dt)

	if spaceship == nil then
		return
	end
	mouse.x, mouse.y = love.mouse.getPosition()
	
	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) < spaceship.angle
	   and math.atan2( mouse.y - gun.y, mouse.x - gun.x) > spaceship.angle-math.pi then
	   
		spaceship.angle = spaceship.angle + - spaceship.rotation_speed*dt
		
		if spaceship.angle < -math.pi then
			spaceship.angle = math.pi
		end
	end

	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) > spaceship.angle 
	   and math.atan2( mouse.y - gun.y, mouse.x - gun.x) < spaceship.angle+math.pi then
		
		spaceship.angle = spaceship.angle + spaceship.rotation_speed*dt
	   
		if spaceship.angle > math.pi then
			spaceship.angle = -math.pi					
		end
	end

	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) > spaceship.angle+math.pi then
		spaceship.angle = spaceship.angle - spaceship.rotation_speed*dt
		
		if spaceship.angle < -math.pi then
			spaceship.angle = math.pi
		end
	end

	if math.atan2( mouse.y - gun.y, mouse.x - gun.x) < spaceship.angle-math.pi then
		
		spaceship.angle = spaceship.angle + spaceship.rotation_speed*dt

		if spaceship.angle > math.pi then
			spaceship.angle = -math.pi
		end
	end
	
	spaceship.hc_shape:setRotation( spaceship.angle )
	
	gun.x = spaceship.x + spaceship_sprite:getWidth()/2 * math.cos( spaceship.angle )
	gun.y = spaceship.y + spaceship_sprite:getWidth()/2 * math.sin( spaceship.angle )
end

start_timer = 0

function update_enemies(dt)

	if start_timer < 3 then
		start_timer = start_timer + dt
		return
	end

	if enemy_timer >= enemy_interval then
		for i=1, math.random(0,10) do
			local enemy = {}
			enemy.x = math.random(0,1024)
			enemy.y = math.random(100,200)*-1
			enemy.size_factor = 1
			--math.random(100,250)/10
			enemy.speed = enemy_default_speed
			enemy.hc_shape = HC.addCircle( enemy.x + enemy.size_factor*enemy_sprite:getWidth()/2,
										   enemy.y + enemy.size_factor*enemy_sprite:getWidth()/2,
										   math.pow(enemy.size_factor, 2)*enemy_sprite:getWidth()/2 )
			enemy.hc_shape.parent = enemy
			enemy.hc_shape.sprite = enemy_sprite
			HC.addToGroup( "enemy_team", enemy.hc_shape )
			table.insert(enemies,enemy)
		end
		enemy_timer = 0
	else
		enemy_timer = enemy_timer+dt
	end
	
	for i=1, #enemies do
		if enemies[i] ~= nil then
			if ( enemies[i].y > 900
				 or enemies[i].hc_shape == nil ) then
				if enemies[i].hc_shape then
					HC.removeFromGroup( "enemy_team", enemies[i].hc_shape )
					HC.remove( enemies[i].hc_shape )
				end
				table.remove(enemies,i)
			else
				enemies[i].y = enemies[i].y+(enemies[i].speed*dt*(enemies[i].size_factor))
				enemies[i].hc_shape:move(0, enemies[i].speed*dt*(enemies[i].size_factor))				
			end
		end
	end
	
	for i=1, #enemies do
		enemies[i].index = i
	end
end

i = 0

function on_collide(dt, shape_a, shape_b)

	i = i+1
		
	if ( shape_a == spaceship.hc_shape and shape_b.sprite == enemy_sprite) then 
		text[#text+1] = string.format("Colliding Spaceship-Enemy %d", i)
		HC.removeFromGroup("enemy_team", shape_b)		
		HC.remove(shape_b)
		shape_b.parent.hc_shape = nil
		spaceship.hp = spaceship.hp-10
		return
	end
	
	if ( shape_b == spaceship.hc_shape and shape_a.sprite == enemy_sprite) then 
		text[#text+1] = string.format("Colliding Enemy-Spaceship %d", i)
		HC.removeFromGroup("enemy_team", shape_a)		
		HC.remove(shape_a)
		shape_a.parent.hc_shape = nil
		spaceship.hp = spaceship.hp-10
		return
	end
	
	if ( shape_a.sprite == enemy_sprite and shape_b.sprite == ammo_sprite) then
		text[#text+1] = string.format("Colliding Enemy-Ammo %d", i)		
		HC.removeFromGroup("enemy_team", shape_a)
		HC.remove(shape_a)
		HC.removeFromGroup("player_team", shape_b)
		HC.remove(shape_b)
		shape_a.parent.hc_shape = nil
		shape_b.parent.hc_shape = nil
		return
	end
	
	if ( shape_b.sprite == enemy_sprite and shape_a.sprite == ammo_sprite) then
		text[#text+1] = string.format("Colliding Ammo-Enemy %d", i)
		HC.removeFromGroup("player_team", shape_a)	
		HC.remove(shape_a)
		HC.removeFromGroup("enemy_team", shape_b)
		HC.remove(shape_b)
		shape_a.parent.hc_shape = nil
		shape_b.parent.hc_shape = nil
		return
	end
end